import { produce } from 'immer';
import { atom, useAtom } from 'jotai';
import { useCallback } from 'react';

/* eslint-disable import/named */
const atomWithImmer = initialValue => {
  const anAtom = atom(initialValue, (get, set, fn) => set(anAtom, produce(get(anAtom), draft => fn(draft))));
  return anAtom;
};

/* eslint-disable import/named */
const useImmerAtom = anAtom => {
  const [state, setState] = useAtom(anAtom);
  const setStateWithImmer = useCallback(fn => {
    setState(produce(draft => fn(draft)));
  }, [setState]);
  return [state, setStateWithImmer];
};

/* eslint-disable import/named */
const withImmer = anAtom => atom(get => get(anAtom), (get, set, fn) => set(anAtom, produce(get(anAtom), draft => fn(draft))));

export { atomWithImmer, useImmerAtom, withImmer };
